@@ -16,8 +16,8 @@
 #    more: https://github.com/manatlan/guy
 # #############################################################################
 
-__version__="0.0.14"
+__version__="0.0.21"
 
-from .gtag import GTag,Tag,local
+from .gtag import GTag, Tag, value, render
 
-__all__ = ['GTag', 'Tag','local'  ]
+__all__ = [ 'GTag', 'Tag', 'value', 'render' ]
