@@ -24,7 +24,7 @@ isAsyncGenerator=lambda x: "async_generator" in str(type(x)) #TODO: howto better
 
 value=lambda x: x.get() if isinstance(x,ReactiveProp) else x
 
-def jjs(obj): #TODO: not optimal ... do better than pattern subst ;-)
+def jjs(obj): #TODO: not optimal ... do better than str's pattern subst ;-)
     """ json dumps (js is b'' (bytes)) """
     def my(obj):
         if isinstance(obj,bytes):
@@ -34,12 +34,12 @@ def jjs(obj): #TODO: not optimal ... do better than pattern subst ;-)
     return guy.json.dumps(obj, default=my).replace('"<:<:',"").replace(':>:>"',"")
 
 def log(*a):
-    #~ print(*a)
+    # print(*a)
     pass
 
 
 class MyMetaclass(type):
-    def __getattr__(self,name:str) -> callable:
+    def __getattr__(self,name:str):
         def _(*a,**k) -> Tag:
             t=Tag(*a,**k)
             t.tag=name
@@ -82,8 +82,6 @@ class Tag(metaclass=MyMetaclass):
     def __repr__(self):
         return "<%s>" % self.__class__.__name__
 
-
-
 class CSS(Tag):
     def __init__(self,content):
         self._md5=hashlib.md5("css:".encode()+content.encode()).hexdigest()
@@ -137,6 +135,11 @@ class ReactiveProp:
         self.set( vv )
         return self
 
+    def __getitem__(self,k):
+        return self.get()[k]
+    def __setitem__(self,k,v):
+        self.get()[k]=value(v)
+
     def __int__(self):
         return int(self.get())
     def __bool__(self):
@@ -147,30 +150,49 @@ class ReactiveProp:
     def __repr__(self):
         iid=self.__instance.id if hasattr(self.__instance,"id") else str(self.__instance)
         return "<%s instance=%s attr=%s>" % (self.__class__.__name__,iid,self.__attribut)
+
+    def __getattr__(self,k):
+        return getattr(self.get(),k)
+    def __setattr__(self,k,v):
+        if k.startswith("_"):
+            super().__setattr__(k, v)
+        else:
+            setattr(self.get(),k,value(v))
+
     #TODO: add a lot of __slot__ ;-)
 
 
+class render:
 
-def local( method ): # gtag.event decorator
-    """ Decorator to make a gtag.method() able to start after init !
-    """
-    Capacity(method).set(Capacity.LOCAL)
-    return method
+    @staticmethod
+    def local( method ): # gtag.event decorator
+        """ Make the method renders only this component (and its childs)"""
+        Capacity(method).set(inspect.getouterframes(inspect.currentframe())[0].function)
+        return method
+
+    @staticmethod
+    def parent( method ): # gtag.event decorator
+        """ Make the method renders only its parent (and its childs) """
+        Capacity(method).set(inspect.getouterframes(inspect.currentframe())[0].function)
+        return method
+
+    @staticmethod
+    def no( method ): # gtag.event decorator
+        """ Make the method renders nothing """
+        Capacity(method).set(inspect.getouterframes(inspect.currentframe())[0].function)
+        return method
 
 class Capacity:
-    LOCAL="local"
     def __init__(self,method:callable):
         self.__method=method
-    def has( self, capacity ):
+    def has( self, f:callable ):
         if hasattr(self.__method,"capacities"):
-            return capacity in self.__method.capacities
+            return f.__name__ in self.__method.capacities
     def set( self, capacity ):
         if not hasattr(self.__method,"capacities"):
             self.__method.capacities=[]
         self.__method.capacities.append(capacity)
-    @property
-    def hasLocal( self ):
-        return self.has(Capacity.LOCAL)
+
 
 
 
@@ -209,7 +231,7 @@ class GTag:
     # implicit parent version (don't need to pass self(=parent) when creating a gtag)
     def __init__(self,*a,**k):
         self._tag=None
-        self.script=None
+        self._scripts=[]
 
         if "parent" in k.keys(): # clonage (only main tags, so parent is None)
             #(but could be used to reparent a gtag manually)
@@ -241,6 +263,8 @@ class GTag:
         self.init(*self._args,**self._kargs)
         self._childs=[]     #<- clear innerchilds (creating during child phase), to avoid to appears in child
 
+        self._scriptsInInit=self._scripts[:]
+
         self._tag = self.build()
 
         # Store the instance in the parent._childs
@@ -250,7 +274,7 @@ class GTag:
     def _tree(self):
         def _gc(g,lvl=0) -> list:
             ll=["+" + ("   "*lvl) + repr(g)]
-            for obj in g.innerChilds:
+            for obj in g._ichilds:
                 inners=_gc(obj,lvl+1)
                 ll.extend( [i+' (INNER)' for i in inners] )
             for obj in g._childs:
@@ -260,14 +284,14 @@ class GTag:
 
 
     @property
-    def innerChilds(self):
+    def _ichilds(self):
         return [v for k,v in self.__dict__.items() if k not in ["_tag","_parent"] and isinstance(v,GTag)]
 
     def _getChilds(self) -> dict:
 
         def _gc(g) -> dict:
             d={g.id:g}
-            for obj in g.innerChilds:
+            for obj in g._ichilds:
                 d.update( _gc(obj) )
             for obj in g._childs:
                 d.update( _gc(obj) )
@@ -277,7 +301,10 @@ class GTag:
 
     def _getRef(self,id): # -> GTag
         childs=self._getChilds()
-        return childs[id]
+        try:
+            return childs[id]
+        except KeyError:
+            raise Exception("ERROR: Unknown child '%s' in '%s'"%(id,self.id))
 
     def _getMain(self):
         x=self
@@ -291,7 +318,7 @@ class GTag:
 
 
     @property
-    def parent(self)-> any: #GTag
+    def parent(self)-> T.Union[GtagProxy,None]:
         """ return caller/binder to parent instance (None if gtag is the main) """
         if self._parent is None:
             return None
@@ -300,14 +327,14 @@ class GTag:
 
 
     @property
-    def main(self)-> any: #GTag
+    def main(self)-> GtagProxy:
         """ return caller/binder to main instance """
         return GtagProxy( self._getMain() )
 
 
 
     @property
-    def bind(self) -> any:
+    def bind(self):
         """ to bind attribute or method !"""
         class Binder:
             def __getattr__(this,name:str):
@@ -329,11 +356,12 @@ class GTag:
         return Binder()
 
 
-    def _clone(self): #TODO: not clear here ... need redone
-        assert self._parent==None,"Can't clone a gtag which is not the main"
+    def _clone(self): #TODO: not clear here ... need redone (the rebuild() needed ?! why ?!)
+        assert self._parent==None,"Can't clone a gtag which is not the main one"
         props={k:v for k,v in self.__dict__.items() if k[0]!="_" or k=="_call"}
         gtag = self.__class__(*self._args,**self._kargs,parent=None) # parent=None, will avoid guess parent ! (it makes sense, because you can clone only mains)
         gtag.__dict__.update(props)
+        gtag._scripts=[]
         gtag.init(*self._args,**self._kargs)
         gtag._rebuild()
         log("^^^ CLONED ^^^",repr(self),"-->",repr(gtag))
@@ -376,12 +404,15 @@ class GTag:
 
     def exit(self,v=None): pass # overriden by run/runcef/serve
 
-    def _rebuild(self):
+    def _clearScripts(self):
+        self._scripts=self._scriptsInInit[:]
+
+    def _rebuild(self,clearScripts=True):
+        if clearScripts: self._clearScripts()
         self._childs=[]
         self._tag=self.build()
 
     def __str__(self):
-
         def getTagIded(o):
             if isinstance(o,Tag):
                 o.id=self.id
@@ -389,7 +420,7 @@ class GTag:
                 o=getTagIded(o._tag)
                 o.id=self.id
             else:
-                o=Tag.span(o)
+                o=Tag.span(o) # auto span'ify
                 o.id=self.id
             return o
 
@@ -398,6 +429,9 @@ class GTag:
         if o is None:
             return ""
         else:
+            for i in self._ichilds:
+                i._rebuild()
+
             o=getTagIded(o)
             return str(o)
 
@@ -405,7 +439,7 @@ class GTag:
         return "<GTag: %s [parent:%s] (innerchilds=%s)>" % (
             self.id,
             self._parent.id if self._parent else "None",
-            [i.id for i in self.innerChilds]
+            [i.id for i in self._ichilds]
         )
 
     def __setattr__(self,k,v):
@@ -415,10 +449,17 @@ class GTag:
         else:
             super().__setattr__(k,v)
 
+    @property
+    def scripts(self):
+        return ";".join(self._scripts)
+
+    def __call__(self,js):
+        self._scripts.append(js)
+
     def _getScripts(self) -> str:
         ll=[]
         for g in self._getChilds().values():
-            js=g.script
+            js=g.scripts
             if js:
                 ll.append( "(function(tag){%s})(document.getElementById('%s'))" % (str(js),g.id) )
         return ";".join(ll)
@@ -436,17 +477,23 @@ class GTag:
     def run(self,*a,start=None,**k) -> any:
         """ Run as Guy App (using Chrome) """
         self._call=start
-        return GTagApp(self,False).run(*a,**k)
+        g=GTagApp(self,False)
+        g._name=self.__class__.__name__
+        return g.run(*a,**k)
 
     def runCef(self,*a,start=None,**k) -> any:
         """ Run as Guy App (using Cef) """
         self._call=start
-        return GTagApp(self,False).runCef(*a,**k)
+        g=GTagApp(self,False)
+        g._name=self.__class__.__name__
+        return GTagApp.runCef(*a,**k)
 
     def serve(self,*a,start=None,**k) -> any:
         """ Run as Guy Server App """
         self._call=start
-        return GTagApp(self,True).serve(*a,**k)
+        g=GTagApp(self,True)
+        g._name=self.__class__.__name__
+        return g.serve(*a,**k)
 
 
 
@@ -476,15 +523,8 @@ class GTagApp(guy.Guy):
         <script>
             if(!sessionStorage["gtag"]) sessionStorage["gtag"]=Math.random().toString(36).substring(2);
             var GID=sessionStorage["gtag"];
-
-            async function forceUpdateManually(x) {console.log(x),eval(x)}
-
             async function getSessionId() {return GID}
-            async function _render(html,script) {
-                document.body.innerHTML=html;
-                if(script) eval(script)
-                console.log(script)
-            }
+            async function render(js) {eval(js)}
         </script>
         %s
     </head>
@@ -504,7 +544,7 @@ class GTagApp(guy.Guy):
         else: # app mode
             gtag = self._originalGTag
 
-        gtag.exit = self.exit
+        gtag.exit = self.exit #plug the exit()
 
         log(">>>SERVE",repr(gtag))
         log(gtag._tree())
@@ -513,6 +553,7 @@ class GTagApp(guy.Guy):
         caller=""
         if gtag._call:  # there is an event to call at start !
             if isAsyncGenerator(gtag._call) or asyncio.iscoroutine(gtag._call):
+                fname,args="",{}
 
                 if asyncio.iscoroutine(gtag._call):
                     fname = gtag._call.__name__
@@ -522,20 +563,18 @@ class GTagApp(guy.Guy):
                     args = gtag._call.ag_frame.f_locals  # dict object
 
                 if "self" in args: del args["self"]
-                args=args.values()
+                args=args.values()  #TODO: how can it works ?! (verify that with TU *a,**k)!)
 
                 method=getattr(gtag.bind,fname)
                 caller=method(*args)
-                print("====>",gtag._call,caller)
         #=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-
-        await self.js._render( str(gtag), gtag._getScripts()+";"+caller )
+        js="document.body.innerHTML=`%s`;%s;%s" %(str(gtag),gtag._getScripts(),caller)
+        await self.js.render(js)
 
     async def forceUpdate(self,g): #can't be called from client side !
-        g._rebuild()
-        js=g._update()["script"]    #TODO: not cute ... should unify way to render to client (currently: 3 ways)
+        g._rebuild(False)
         log(">>>Force UPDATE:",repr(g))
-        await self.js.forceUpdateManually(js)
+        await self.js.render( g._update()["script"] )
 
     async def bindUpdate(self,id:str,gid:str,method:str,args,kargs):
         """ inner (js exposed) guy method, called by gtag.bind.<method>(*args) """
@@ -549,7 +588,17 @@ class GTagApp(guy.Guy):
 
         log("BINDUPDATE on",repr(gtag),"----->",repr(obj),"%s(%s %s)"% (method,args,kargs))
         proc=getattr(obj,method)
-        toRender=gtag if not Capacity(proc).hasLocal else obj
+
+        if Capacity(proc).has(render.local):
+            toRender=obj
+        elif Capacity(proc).has(render.parent):
+            toRender=obj._parent if obj._parent else obj
+        elif Capacity(proc).has(render.no):
+            toRender=None
+        else:
+            toRender=gtag
+
+        if toRender: toRender._clearScripts()
 
         if asyncio.iscoroutinefunction( proc ):
             rep=await proc(*args,**kargs)
@@ -559,10 +608,14 @@ class GTagApp(guy.Guy):
         if rep:
             if isAsyncGenerator(rep):
                 async for _ in rep: # could use yielded thing to update all or local ?!
-                    await self.forceUpdate(toRender)
+                    assert _ is None, "wtf?"
+                    if toRender:
+                        await self.forceUpdate(toRender)
             else:
                 raise Exception("wtf?")
 
-        toRender._rebuild()
-        return toRender._update() #UPDATE ALL (historic way)
+        if toRender:
+            toRender._rebuild(False)
+            return toRender._update() #UPDATE ALL (historic way)
         #////////////////////////////////////////////////////////////////// THE MAGIC
+
